@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (C) 2010-2023 Bastian Kleineidam
 #
 # This program is free software: you can redistribute it and/or modify
@@ -14,79 +13,72 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Utility functions."""
+
+import functools
 import os
+import sys
 import shutil
 import subprocess
+from collections.abc import Sequence
 from .log import log_info
 
 
-class PatoolError (Exception):
+class PatoolError(Exception):
     """Raised when errors occur."""
+
     pass
 
 
-class memoized (object):
-    """Decorator that caches a function's return value each time it is called.
-    If called later with the same arguments, the cached value is returned, and
-    not re-evaluated."""
-
-    def __init__(self, func):
-        """Set func and init cache."""
-        self.func = func
-        self.cache = {}
-
-    def __call__(self, *args):
-        """Try to find result for function arguments in local cache or
-        execute the function and fill the cache with the result."""
-        try:
-            return self.cache[args]
-        except KeyError:
-            self.cache[args] = value = self.func(*args)
-            return value
-        except TypeError:
-            # uncachable -- for instance, passing a list as an argument.
-            # Better to not cache than to blow up entirely.
-            return self.func(*args)
-
-    def __repr__(self):
-        """Return the function's docstring."""
-        return self.func.__doc__
-
-
-def backtick(cmd, encoding='utf-8'):
+def backtick(cmd: Sequence[str], encoding: str = 'utf-8') -> str:
     """Return decoded output from command."""
-    return subprocess.run(cmd, stdout=subprocess.PIPE, check=True,
-                          encoding=encoding, errors="replace").stdout
+    return subprocess.run(
+        cmd, stdout=subprocess.PIPE, check=True, encoding=encoding, errors="replace"
+    ).stdout
 
 
-def run(cmd, verbosity=0, **kwargs):
+def run_under_pythonw() -> bool:
+    """Return True iff this script is run with pythonw.exe on Windows."""
+    return (
+        os.name == 'nt'
+        and sys.executable is not None
+        and sys.executable.lower().endswith('pythonw.exe')
+    )
+
+
+def run(cmd: Sequence[str], verbosity: int = 0, **kwargs) -> int:
     """Run command without error checking.
-    @return: command return code"""
+    @return: command return code
+    """
     # Note that shell_quote_nt() result is not suitable for copy-paste
     # (especially on Unix systems), but it looks nicer than shell_quote().
     if verbosity >= 0:
         info = " ".join(map(shell_quote_nt, cmd))
         log_info(f"running {info}")
-    if os.name == "nt":
+    if run_under_pythonw():
         # prevent opening of additional consoles when running with pythonw.exe
-        kwargs["creationflags"] = subprocess.CREATE_NO_WINDOW
+        kwargs["creationflags"] = (
+            subprocess.CREATE_NO_WINDOW  # pytype: disable=module-attr
+        )
     # try to prevent hangs for programs requiring input
     kwargs["input"] = ""
+    if verbosity < 1:
+        # hide command output on stdout
+        kwargs['stdout'] = subprocess.DEVNULL
+    if verbosity < -1:
+        # hide command output on stdout
+        kwargs['stderr'] = subprocess.DEVNULL
     if kwargs:
-        if verbosity >= 0:
+        if verbosity > 0:
             info = ", ".join(f"{k}={shell_quote(str(v))}" for k, v in kwargs.items())
             log_info(f"    with {info}")
         if kwargs.get("shell"):
             # for shell calls the command must be a string
             cmd = " ".join(cmd)
-    if verbosity < 1:
-        # hide command output on stdout
-        kwargs['stdout'] = subprocess.DEVNULL
     res = subprocess.run(cmd, **kwargs)
     return res.returncode
 
 
-def run_checked(cmd, ret_ok=(0,), **kwargs):
+def run_checked(cmd: Sequence[str], ret_ok: Sequence[int] = (0,), **kwargs) -> int:
     """Run command and raise PatoolError on error."""
     retcode = run(cmd, **kwargs)
     if retcode not in ret_ok:
@@ -95,29 +87,31 @@ def run_checked(cmd, ret_ok=(0,), **kwargs):
     return retcode
 
 
-def shell_quote(value):
+def shell_quote(value: str) -> str:
     """Quote all shell metacharacters in given string value with strong
-    (i.e. single) quotes, handling the single quote especially."""
+    (i.e. single) quotes, handling the single quote especially.
+    """
     if os.name == 'nt':
         return shell_quote_nt(value)
     return shell_quote_unix(value)
 
 
-def shell_quote_unix(value):
+def shell_quote_unix(value: str) -> str:
     """Quote argument for Unix system."""
     value = value.replace("'", r"'\''")
     return f"'{value}'"
 
 
-def shell_quote_nt(value):
+def shell_quote_nt(value: str) -> str:
     """Quote argument for Windows system. Modeled after distutils
-    _nt_quote_args() function."""
+    _nt_quote_args() function.
+    """
     if " " in value:
         return f'"{value}"'
     return value
 
 
-def p7zip_supports_rar():
+def p7zip_supports_rar() -> bool:
     """Determine if the RAR codec is installed for 7z program."""
     if os.name == 'nt':
         # Assume RAR support is compiled into the binary.
@@ -125,7 +119,14 @@ def p7zip_supports_rar():
     # the subdirectory and codec name
     codecnames = ['p7zip/Codecs/Rar29.so', 'p7zip/Codecs/Rar.so']
     # search canonical user library dirs
-    for libdir in ('/usr/lib', '/usr/local/lib', '/usr/lib64', '/usr/local/lib64', '/usr/lib/i386-linux-gnu', '/usr/lib/x86_64-linux-gnu'):
+    for libdir in (
+        '/usr/lib',
+        '/usr/local/lib',
+        '/usr/lib64',
+        '/usr/local/lib64',
+        '/usr/lib/i386-linux-gnu',
+        '/usr/lib/x86_64-linux-gnu',
+    ):
         for codecname in codecnames:
             fname = os.path.join(libdir, codecname)
             if os.path.exists(fname):
@@ -133,7 +134,7 @@ def p7zip_supports_rar():
     return False
 
 
-def system_search_path():
+def system_search_path() -> str:
     """Get the list of directories to search for executable programs."""
     path = os.environ.get("PATH", os.defpath)
     if os.name == 'nt':
@@ -144,9 +145,10 @@ def system_search_path():
     return path
 
 
-def append_to_path(path, directory):
+def append_to_path(path: str, directory: str) -> str:
     """Add a directory to the PATH environment variable, if it is a valid
-    directory."""
+    directory.
+    """
     if not os.path.isdir(directory) or directory in path:
         return path
     if not path.endswith(os.pathsep):
@@ -154,50 +156,55 @@ def append_to_path(path, directory):
     return path + directory
 
 
-@memoized
-def find_program(program):
+@functools.cache
+def find_program(program: str) -> str | None:
     """Look for given program."""
     return shutil.which(program, path=system_search_path())
 
 
-def get_nt_7z_dir():
+def get_nt_7z_dir() -> str:
     """Return 7-Zip directory from registry, or an empty string."""
-    import winreg
+    import winreg  # type: ignore
     import platform
+
     python_bits = platform.architecture()[0]
     keyname = r"SOFTWARE\7-Zip"
     try:
         if python_bits == '32bit' and platform.machine().endswith('64'):
             # get 64-bit registry key from 32-bit Python
-            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, keyname,
-                  0, winreg.KEY_READ | winreg.KEY_WOW64_64KEY)
+            key = winreg.OpenKey(
+                winreg.HKEY_LOCAL_MACHINE,
+                keyname,
+                0,
+                winreg.KEY_READ | winreg.KEY_WOW64_64KEY,
+            )
         else:
             key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, keyname)
         try:
             return winreg.QueryValueEx(key, "Path")[0]
         finally:
             winreg.CloseKey(key)
-    except WindowsError:
+    except OSError:
         return ""
 
 
-def get_nt_program_dir ():
+def get_nt_program_dir() -> str:
     """Return the Windows program files directory."""
     progvar = "%ProgramFiles%"
     return os.path.expandvars(progvar)
 
 
-def get_nt_mac_dir ():
+def get_nt_mac_dir() -> str:
     """Return Monkey Audio Compressor (MAC) directory."""
     return os.path.join(get_nt_program_dir(), "Monkey's Audio")
 
 
-def get_nt_winrar_dir():
+def get_nt_winrar_dir() -> str:
     """Return WinRAR directory."""
     return os.path.join(get_nt_program_dir(), "WinRAR")
 
 
-def strlist_with_or(alist):
+def strlist_with_or(alist: Sequence[str]) -> str:
     """Return comma separated string, and last entry appended with ' or '."""
     if len(alist) > 1:
         head = ", ".join(alist[:-1])
